@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #  _  __
-# | |/ /___ ___ _ __  ___ _ _ ®
+# | |/ /___ ___ _ __  ___ _ _ (R)
 # | ' </ -_) -_) '_ \/ -_) '_|
 # |_|\_\___\___| .__/\___|_|
 #              |_|
@@ -18,6 +18,7 @@ from . import KeeperCli
 from .exception import KsmCliException
 from .exec import Exec
 from .secret import Secret
+from .sync import Sync
 from .profile import Profile
 from .init import Init
 from .config import Config
@@ -31,12 +32,12 @@ import typing as t
 from update_checker import UpdateChecker
 
 
-global_config = None
+global_config = Config()
 
 
 # NOTE: For the CLI, all groups and command are lowercase. All arguments are lower case, so you cannot use
-# -n and -N for an arg flag. If you add a command, you need to add it to the list of known commands so we can
-# do a best match.
+# -n and -N for an arg flag. If you add a command, you need to add it to the list of known commands, so we can
+# do a "best match."
 
 
 class AliasedGroup(HelpColorsGroup):
@@ -50,6 +51,7 @@ class AliasedGroup(HelpColorsGroup):
         "export",
         "import",
         "init",
+        "sync",
         "secret",
         "totp",
         "download",
@@ -82,6 +84,7 @@ class AliasedGroup(HelpColorsGroup):
         "n": "notation",
         "u": "update",
         "v": "version",
+        "y": "sync",
         "exit": "quit",
         "pass": "password",
         "q": "quit",
@@ -150,7 +153,7 @@ def get_versions():
         "keeper-secrets-manager-cli": "Unknown"
     }
 
-    # Inside of the binaries, it's hard to get versions # so we create a versions.txt file in the build.
+    # In the binaries, it's hard to get versions # so we create a versions.txt file in the build.
     # If the versions.txt file exists, read the versions from that file.
 
     ksm_bin_path = os.path.dirname(__file__)
@@ -217,6 +220,45 @@ def base_command_help(f):
     return f
 
 
+class Mutex(click.Option):
+    def __init__(self, *args, **kwargs):
+        # Detect mutually exclusive or required options - search by key only or key and value
+        self.required_if:t.List[t.Tuple[str,str]] = kwargs.pop("required_if", [])
+        self.not_required_if:t.List[t.Tuple[str,str]] = kwargs.pop("not_required_if", [])
+
+        # at least one search parameter is required
+        assert self.required_if or self.not_required_if, "'required_if' and/or 'not_required_if' parameter required"
+
+        # if both params present they shouldn't overlap
+        if self.required_if and self.not_required_if:
+            overlap = [x for x in self.required_if if x in self.not_required_if]
+            assert not overlap, "'required_if' and 'not_required_if' parameters should not overlap in " + ", ".join("(%s)" % ",".join(x) for x in overlap)
+
+        exclusive_msg = ("Option is mutually exclusive with " + ", ".join("(%s)" % ",".join(tup) for tup in self.not_required_if) + ".") if self.not_required_if else ""
+        required_msg  = ("Option is required with " + ", ".join("(%s)" % ",".join(tup) for tup in self.required_if) + ".") if self.required_if else ""
+        kwargs["help"] = (kwargs.get("help", "") + " " + required_msg + " " + exclusive_msg).strip()
+
+        super(Mutex, self).__init__(*args, **kwargs)
+
+    def handle_parse_result(self, ctx, opts, args):
+        # ('option','value') - option present with the specified value assigned
+        # ('option',) - option present with or without any value
+        current_opt:bool = self.name in opts
+        for mutex_opt in self.not_required_if:
+            if mutex_opt and mutex_opt[0] in opts and (len(mutex_opt) == 1 or opts.get(mutex_opt[0], str(mutex_opt[1])+'_') == mutex_opt[1]):
+                if current_opt:
+                    raise click.UsageError("Illegal usage: '" + str(self.name) + "' is mutually exclusive with " + str(mutex_opt) + ".")
+                else:
+                    self.prompt = None
+        for mutex_opt in self.required_if:
+            if mutex_opt and mutex_opt[0] in opts and (len(mutex_opt) == 1 or opts.get(mutex_opt[0], str(mutex_opt[1])+'_') == mutex_opt[1]):
+                if not current_opt:
+                    raise click.UsageError("Illegal usage: '" + str(self.name) + "' is required with " + str(mutex_opt) + ".")
+                else:
+                    self.prompt = None
+        return super(Mutex, self).handle_parse_result(ctx, opts, args)
+
+
 # MAIN GROUP
 @click.group(
     cls=AliasedGroup,
@@ -228,9 +270,10 @@ def base_command_help(f):
 @click.option('--output', '-o', type=str, help='Output [stdout|stderr|filename]', default='stdout')
 @click.option('--color/--no-color', '-c/-nc', default=None, help="Use color in table views, where applicable.")
 @click.option('--cache/--no-cache', default=None, help="Enable/disable record caching.")
+@click.option('--log-level', type=click.Choice(['DEBUG', 'INFO', 'WARNING', 'ERROR']), help="Debug log level.")
 @click.pass_context
 @base_command_help
-def cli(ctx, ini_file, profile_name, output, color, cache):
+def cli(ctx, ini_file, profile_name, output, color, cache, log_level):
     """Keeper Secrets Manager CLI
     """
 
@@ -241,12 +284,15 @@ def cli(ctx, ini_file, profile_name, output, color, cache):
             output=output,
             use_color=color,
             use_cache=cache,
-            global_config=global_config),
+            global_config=global_config,
+            log_level=log_level
+        ),
         "ini_file": ini_file,
         "profile_name": profile_name,
         "output": output,
         "use_color": color,
-        "use_cache": cache
+        "use_cache": cache,
+        "log_level": log_level
     }
 
 
@@ -292,7 +338,8 @@ def profile_init_command(ctx, token, hostname, ini_file, profile_name, token_arg
         token=token,
         server=hostname,
         ini_file=ini_file,
-        profile_name=profile_name
+        profile_name=profile_name,
+        launched_from_app=global_config.launched_from_app
     )
 
 
@@ -358,7 +405,8 @@ def profile_import_command(ctx, output_file, config_base64):
     """Import an encrypted config file"""
     Profile(cli=ctx.obj["cli"], config=global_config).import_config(
         file=output_file,
-        config_base64=config_base64
+        config_base64=config_base64,
+        launched_from_app=global_config.launched_from_app
     )
 
 
@@ -419,9 +467,10 @@ def secret_list_command(ctx, uid, json):
 @click.option('--force-array', is_flag=True, help="Return secrets as array even if a single record.")
 @click.option('--unmask', is_flag=True, help="Show password like values in table views.")
 @click.option('--inflate/--deflate', default=True, help="Load in outside records.")
+@click.option('--raw', is_flag=True, help='Remove surrounding quotes on value when using query.')
 @click.argument('extra-uid', type=str, nargs=-1)
 @click.pass_context
-def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask, inflate, extra_uid):
+def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask, inflate, extra_uid, raw):
     """Get secret record(s)"""
 
     uid_list = []
@@ -454,7 +503,8 @@ def secret_get_command(ctx, uid, title, field, query, json, force_array, unmask,
         load_references=True,
         unmask=unmask,
         use_color=ctx.obj["cli"].use_color,
-        inflate=inflate
+        inflate=inflate,
+        raw=raw
     )
 
 
@@ -953,7 +1003,7 @@ def version_command(ctx):
     print("CLI Install: {}".format(os.path.dirname(os.path.realpath(__file__))))
     print("SDK Version: {}".format(versions["keeper-secrets-manager-core"]))
     print("SDK Install: {}".format(os.path.dirname(os.path.realpath(keeper_secrets_manager_core.__file__))))
-    print("Config file: {}".format(ctx.obj["cli"].profile.ini_file))
+    print("Config file: {}".format(global_config.ini_file))
 
     try:
         if versions["keeper-secrets-manager-cli"] != "Unknown":
@@ -974,12 +1024,16 @@ def version_command(ctx):
     cls=HelpColorsCommand,
     help_options_color='blue'
 )
-def shell_command():
+@click.option('--app', is_flag=True, help='Launched from application.')
+def shell_command(app):
     """Run KSM in a shell"""
 
     global global_config
     global_config = Config()
 
+    # Flag that the application was launched as a Windows/macOS application.
+    global_config.launched_from_app = app
+
     # https://manytools.org/hacker-tools/ascii-banner/
     logo = """
 ██╗  ██╗███████╗███╗   ███╗     ██████╗██╗     ██╗
@@ -1016,8 +1070,32 @@ def quit_command():
     repl_exit()
 
 
+# SYNC COMMAND
+
+@click.command(
+    name='sync',
+    cls=HelpColorsCommand,
+    help_options_color='blue'
+)
+@click.option('--credentials', '-c', type=str, metavar="UID", help="Keeper record with credentials to access destination key/value store.",
+    cls=Mutex,
+    # not_required_if=[('type','json')],
+    required_if=[('type','azure'), ('type','aws')]
+)
+@click.option('--type', '-t', type=click.Choice(['aws', 'azure', 'json']), default='json', help="Type of the target key/value storage (aws, azure, json).", show_default=True)
+@click.option('--dry-run', '-n', is_flag=True, help='Perform a trial run with no changes made.')
+@click.option('--preserve-missing', '-p', is_flag=True, help='Preserve destination value when source value is deleted.')
+@click.option('--map', '-m', nargs=2, type=(str, str), multiple=True, required=True, metavar="<KEY NOTATION>...", help='Map destination key names to values using notation URI.')
+@click.pass_context
+def sync_command(ctx, credentials, type, dry_run, preserve_missing, map):
+    """Sync selected keys from Keeper vault to secure cloud based key value store"""
+    sync = Sync(cli=ctx.obj["cli"])
+    sync.sync_values(type=type, credentials=credentials, dry_run=dry_run, preserve_missing=preserve_missing, map=map)
+
+
 # TOP LEVEL COMMANDS
 cli.add_command(profile_command)
+cli.add_command(sync_command)
 cli.add_command(secret_command)
 cli.add_command(exec_command)
 cli.add_command(config_command)
